;============================================================
;Checks current level file and compares to entries in INI file.
;============================================================
;CALLED BEFORE EVERY LEVEL START/AFTER EVERY LEVEL LOAD
;(called after calls near "Finished Loading!" string reference)
;NEW: also includes CAnimal instance modification code (dino randomizer, etc.)
;Loads if same file name found.
;NOTE: CStartTriggers are played BEFORE this code.
;-> see "SAVEMAPSTATE.inc" for short description of SAVEDMAP.xxx file format

;[ebp-4] = file handle
;[ebp-8] = file mapping handle (if used/not saved game)
;ebx = EOF marker

LOADINITIALBITMAP:
add esp,(604h+4)		;REPLACED CODE (modified to include return address)
push DWORD [esp-(604h+4)]	;move return address

push ebp
mov ebp,esp
sub esp,8
push ebx
push ecx
push edx
push esi
push edi
push eax

mov DWORD [ebp-4],0
mov DWORD [ebp-8],0
;================================================================
	mov esi,[LEVELNAME_OBJ] 	;(esi points to level SCN name)
	add esi,20			;(esi points to savegame/scn file name and path)
	mov edi,SAVEGAMENAME_BUFFER	;PATH+NAME OF CURRENT LEVEL FILE COPY (OR SAVEGAME)
	call TransferString

	call IsLevelSavedGame
	test eax,eax
	     jz STANDARD_SCN_FILE

;================================================================
;------------- assume it is otherwise a saved game (LOAD SAVEDMAP.xxx AND ALL COMPONENTS)
;================================================================
	mov DWORD [edi-8],'DMAP'	;possible save extension saved as 'SAVEDMAP.xxx'

invoke CreateFile,SAVEGAMENAME_BUFFER,GENERIC_READ,0,0,\
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0
	test eax,eax
	     jz SAVED_FILE_LOAD_FAIL
	cmp eax,-1
	    jz SAVED_FILE_LOAD_FAIL
	mov [ebp-4],eax

invoke GetFileSize,eax,0
	cmp eax,-1
	    jz SAVED_FILE_LOAD_FAIL
	cmp eax,21		;at least 4 32-bit floats + 'x.bmp'
	    jb SAVED_FILE_LOAD_FAIL

invoke SetFilePointer,DWORD [ebp-4],0,0,FILE_BEGIN
	cmp eax,-1
	    jz SAVED_FILE_LOAD_FAIL

;----------------------------------
;read 4 POSITION float values and store
;----------------------------------
	lea eax,[ebp-8]
invoke ReadFile,DWORD [ebp-4],SAVED_MAP_X1,16,eax,0
	test eax,eax
	     jz SAVED_FILE_LOAD_FAIL
	cmp DWORD [ebp-8],16
	    jnz SAVED_FILE_LOAD_FAIL

;----------------------------------
;read BITMAP name and store
;----------------------------------
	lea eax,[ebp-8]
invoke ReadFile,DWORD [ebp-4],SAVED_MAP_BUFFER,80,eax,0
	test eax,eax
	     jz SAVED_FILE_LOAD_FAIL
	cmp DWORD [ebp-8],0
	    jna SAVED_FILE_LOAD_FAIL

;----------------------------------
	cmp DWORD [ebp-8],80
	    jnz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD

;-----------------------------------
;Read saved game speed
;-----------------------------------

	lea eax,[ebp-8]
invoke ReadFile,DWORD [ebp-4],stack_ext_buffer,4,eax,0
	test eax,eax
	     jz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD
	cmp DWORD [ebp-8],4
	    jnz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD


	cmp [OverrideSVGSpeed],FALSE
	    jnz OverrideSVGSpeed_OK
	mov eax,DWORD [stack_ext_buffer]
	mov edx,[GameSpeedVarADDR]
	mov [NormalGameSpeed],eax
	mov [edx],eax					 ;set initial game speed
	OverrideSVGSpeed_OK:

;-----------------------------------
;Read AT106 Hint String and set initial hint
;-----------------------------------

	lea eax,[ebp-8]
invoke ReadFile,DWORD [ebp-4],stack_ext_buffer,4,eax,0	 ;get string length
	test eax,eax
	     jz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD
	cmp DWORD [ebp-8],4
	    jnz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD

	mov edi,DWORD [stack_ext_buffer]
	cmp edi,1
	    jb NoSAVEDMAPLOAD_GAMESPEED_HINTMOD
	invoke LocalAlloc,LMEM_FIXED,edi		 ;allocate mem for string
	test eax,eax
	mov ebx,eax
	     jz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD

	lea eax,[ebp-8]
invoke ReadFile,DWORD [ebp-4],ebx,edi,eax,0		 ;get string
	test eax,eax
	     jz NoSAVEDMAPLOAD_GAMESPEED_HINTMOD
	cmp DWORD [ebp-8],edi
	    jz AT106StringReadFromSAVEDMAP_OK
	    invoke LocalFree,ebx
	    jmp NoSAVEDMAPLOAD_GAMESPEED_HINTMOD
AT106StringReadFromSAVEDMAP_OK:

	LOCALRELEASE [HINT_NEW_STRING_ADDR]
	mov [HINT_NEW_STRING_ADDR],ebx
	mov eax,[CURRENT_HINT_ID_ADDR]
	mov [eax],DWORD NEW_HINT_TYPE_ID		 ;set new initial Hint string for level

NoSAVEDMAPLOAD_GAMESPEED_HINTMOD:

;================================================================
;----------------- LOAD BITMAP FROM SAVEDMAP.xxx
;================================================================

	mov ecx,SAVED_MAP_OBJECT
	call AT102MAIN
	test eax,eax		;if load fails, use level's originally indexed map
	     jnz End_LOADBMP_345345	  ;(jumps if success)

SAVED_FILE_LOAD_FAIL:

;================================================================
STANDARD_SCN_FILE:
;================================================================
;scan for bitmap name in/from level name section; if found, scan for X/Y position values

cmp [MapLoadSuccess_BOOL],TRUE		 ;Has a map already been successfully loaded via CStartTrigger?
    jz End_LOADBMP_345345

	mov edi,MapsDirBuffer
	mov esi,MapIndex		 ;INI filename
	add edi,[MapsPathSize]
	call TransferString

	mov esi,[LEVELNAME_OBJ]  ;addr of SCN level name string (section)

	mov DWORD [AT102EMULATEDOBJECT+8],BMPMAPNAME_BUF
invoke GetPrivateProfileString,esi,STRING_Bitmap,STRING_BMPDEF,BMPMAPNAME_BUF,79,MapsDirBuffer
	cmp DWORD [BMPMAPNAME_BUF],00464544h	       ;default string "DEF",0
	    jz NEW_BITMAP_LOAD_FAIL

invoke GetPrivateProfileFloat,esi,STRING_X1,ZERO_STRING,MapsDirBuffer
	mov [AT102EMULATEDOBJECT+12],eax
invoke GetPrivateProfileFloat,esi,STRING_Y1,ZERO_STRING,MapsDirBuffer
	mov [AT102EMULATEDOBJECT+16],eax
invoke GetPrivateProfileFloat,esi,STRING_X2,ZERO_STRING,MapsDirBuffer
	mov [AT102EMULATEDOBJECT+20],eax
invoke GetPrivateProfileFloat,esi,STRING_Y2,ZERO_STRING,MapsDirBuffer
	mov [AT102EMULATEDOBJECT+24],eax

	mov ecx,[MapsPathSize]
	mov BYTE [ecx+MapsDirBuffer],0

;================================================================
;------ Load new bitmap (if not saved game)
;================================================================
	mov ecx,AT102EMULATEDOBJECT
	call AT102MAIN		;emulates a normal AT102 call to load file
	     test eax,eax
		  jnz End_LOADBMP_345345	;(jump if success)
;================================================================
NEW_BITMAP_LOAD_FAIL:
	LOCALRELEASE [currentbitmap]		;unload old map if existing
;================================================================
End_LOADBMP_345345:

	CLOSEHANDLE DWORD [ebp-4]

;================================================================
	call Modify_External
;================================================================

pop eax
pop edi
pop esi
pop edx
pop ecx
pop ebx
mov esp,ebp
pop ebp
ret
